# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from enum import IntEnum


class KtxTranscodeFmt(IntEnum):
    """Enumerators for specifying the transcode target format."""

    ETC1_RGB = 0
    ETC2_RGBA = 1
    BC1_RGB = 2
    BC3_RGBA = 3
    BC4_R = 4
    BC5_RG = 5
    BC7_RGBA = 6
    PVRTC1_4_RGB = 8
    PVRTC1_4_RGBA = 9
    ASTC_4x4_RGBA = 10
    PVRTC2_4_RGB = 18
    PVRTC2_4_RGBA = 19
    ETC2_EAC_R11 = 20
    ETC2_EAC_RG11 = 21
    RGBA32 = 13
    RGB565 = 14
    BGR565 = 15
    RGBA4444 = 16
    ETC = 22
    BC1_OR_3 = 23
    NO_SELECTION = 0x7fffffff



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

import re
from dataclasses import dataclass
from .ktx_pack_astc_block_dimension import KtxPackAstcBlockDimension
from .ktx_pack_astc_encoder_mode import KtxPackAstcEncoderMode
from .ktx_pack_astc_quality_levels import KtxPackAstcQualityLevels

swizzle_regex = re.compile('^[rgba01]{4}$')


@dataclass
class KtxAstcParams:
    """Data for passing extended parameters to KtxTexture2.compress_astc()."""

    verbose: bool = False
    """If true, prints Astc encoder operation details to stdout. Not recommended for GUI apps."""

    thread_count: int = 1
    """Number of threads used for compression. Default is 1."""

    block_dimension: KtxPackAstcBlockDimension = KtxPackAstcBlockDimension.D4x4
    """Combinations of block dimensions that astcenc supports i.e. 6x6, 8x8, 6x5 etc"""

    mode: KtxPackAstcEncoderMode = KtxPackAstcEncoderMode.DEFAULT
    """Can be {ldr/hdr} from astcenc"""

    quality_level: int = KtxPackAstcQualityLevels.FASTEST
    """astcenc supports -fastest, -fast, -medium, -thorough, -exhaustive"""

    normal_map: bool = False
    """
    Tunes codec parameters for better quality on normal maps.

    In this mode normals are compressed to X,Y components
    Discarding Z component, reader will need to generate Z
    component in shaders.
    """

    perceptual: bool = False
    """
    The codec should optimize for perceptual error, instead of direct RMS error.

    This aims to improves perceived image quality, but
    typically lowers the measured PSNR score. Perceptual methods are
    currently only available for normal maps and RGB color data.
    """

    input_swizzle: bytes = bytes([0, 0, 0, 0])
    """
    A swizzle to provide as input to astcenc.

    It must match the regular expression /^[rgba01]{4}$/.
    """

    def parse_swizzle(self, swizzle: str) -> None:
        if not swizzle_regex.match(swizzle):
            raise ValueError(swizzle
                             + " does not match the appropriate format "
                             + str(swizzle_regex)
                             + " for a swizzle")

        self.input_swizzle = swizzle.encode('ascii')



# Copyright (c) 2021, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from enum import IntEnum


class GlInternalformat(IntEnum):
    """OpenGL internalformat constants."""

    R3_G3_B2 = 0x2A10
    RGB4 = 0x804F
    RGB5 = 0x8050
    RGB8 = 0x8051
    RGB10 = 0x8052
    RGB12 = 0x8053
    RGB16 = 0x8054
    RGBA2 = 0x8055
    RGBA4 = 0x8056
    RGB5_A1 = 0x8057
    RGBA8 = 0x8058
    RGB10_A2 = 0x8059
    RGBA12 = 0x805A
    RGBA16 = 0x805B

    COMPRESSED_RGB_S3TC_DXT1_EXT = 0x83F0
    COMPRESSED_RGBA_S3TC_DXT1_EXT = 0x83F1
    COMPRESSED_RGBA_S3TC_DXT3_EXT = 0x83F2
    COMPRESSED_RGBA_S3TC_DXT5_EXT = 0x83F3

    SRGB_EXT = 0x8C40
    SRGB8_EXT = 0x8C41
    SRGB_ALPHA_EXT = 0x8C42
    SRGB8_ALPHA8_EXT = 0x8C43
    SLUMINANCE_ALPHA_EXT = 0x8C44
    SLUMINANCE8_ALPHA8_EXT = 0x8C45
    SLUMINANCE_EXT = 0x8C46
    SLUMINANCE8_EXT = 0x8C47
    COMPRESSED_SRGB_EXT = 0x8C48
    COMPRESSED_SRGB_ALPHA_EXT = 0x8C49
    COMPRESSED_SLUMINANCE_EXT = 0x8C4A
    COMPRESSED_SLUMINANCE_ALPHA_EXT = 0x8C4B
    COMPRESSED_SRGB_S3TC_DXT1_EXT = 0x8C4C
    COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT = 0x8C4D
    COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT = 0x8C4E
    COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT = 0x8C4F

    COMPRESSED_RGB_PVRTC_4BPPV1_IMG = 0x8C00
    COMPRESSED_RGB_PVRTC_2BPPV1_IMG = 0x8C01
    COMPRESSED_RGBA_PVRTC_4BPPV1_IMG = 0x8C02
    COMPRESSED_RGBA_PVRTC_2BPPV1_IMG = 0x8C03

    ATC_RGB_AMD = 0x8C92
    ATC_RGBA_EXPLICIT_ALPHA_AMD = 0x8C93
    ATC_RGBA_INTERPOLATED_ALPHA_AMD = 0x87EE

    COMPRESSED_LUMINANCE_LATC1_EXT = 0x8C70
    COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT = 0x8C72
    COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT = 0x8C71
    COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT = 0x8C73

    ETC1_RGB8_OES = 0x8D64

    COMPRESSED_RGBA_BPTC_UNORM_EXT = 0x8E8C
    COMPRESSED_SRGB_ALPHA_BPTC_UNORM_ARB = 0x8E8D
    COMPRESSED_RGB_BPTC_SIGNED_FLOAT_ARB = 0x8E8E
    COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT_ARB = 0x8E8F

    COMPRESSED_RGB8_ETC2 = 0x9274
    COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2 = 0x9276
    COMPRESSED_RGBA8_ETC2_EAC = 0x9278
    COMPRESSED_SRGB8_ALPHA8_ETC2_EAC = 0x9279
    COMPRESSED_R11_EAC = 0x9270
    COMPRESSED_SIGNED_R11_EAC = 0x9271
    COMPRESSED_RG11_EAC = 0x9272
    COMPRESSED_SIGNED_RG11_EAC = 0x9273

    COMPRESSED_RGBA_ASTC_4x4_KHR = 0x93B0
    COMPRESSED_RGBA_ASTC_5x4_KHR = 0x93B1
    COMPRESSED_RGBA_ASTC_5x5_KHR = 0x93B2
    COMPRESSED_RGBA_ASTC_6x5_KHR = 0x93B3
    COMPRESSED_RGBA_ASTC_6x6_KHR = 0x93B4
    COMPRESSED_RGBA_ASTC_8x5_KHR = 0x93B5
    COMPRESSED_RGBA_ASTC_8x6_KHR = 0x93B6
    COMPRESSED_RGBA_ASTC_8x8_KHR = 0x93B7
    COMPRESSED_RGBA_ASTC_10x5_KHR = 0x93B8
    COMPRESSED_RGBA_ASTC_10x6_KHR = 0x93B9
    COMPRESSED_RGBA_ASTC_10x8_KHR = 0x93BA
    COMPRESSED_RGBA_ASTC_10x10_KHR = 0x93BB
    COMPRESSED_RGBA_ASTC_12x10_KHR = 0x93BC
    COMPRESSED_RGBA_ASTC_12x12_KHR = 0x93BD
    COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR = 0x93D0
    COMPRESSED_SRGB8_ALPHA8_ASTC_5x4_KHR = 0x93D1
    COMPRESSED_SRGB8_ALPHA8_ASTC_5x5_KHR = 0x93D2
    COMPRESSED_SRGB8_ALPHA8_ASTC_6x5_KHR = 0x93D3
    COMPRESSED_SRGB8_ALPHA8_ASTC_6x6_KHR = 0x93D4
    COMPRESSED_SRGB8_ALPHA8_ASTC_8x5_KHR = 0x93D5
    COMPRESSED_SRGB8_ALPHA8_ASTC_8x6_KHR = 0x93D6
    COMPRESSED_SRGB8_ALPHA8_ASTC_8x8_KHR = 0x93D7
    COMPRESSED_SRGB8_ALPHA8_ASTC_10x5_KHR = 0x93D8
    COMPRESSED_SRGB8_ALPHA8_ASTC_10x6_KHR = 0x93D9
    COMPRESSED_SRGB8_ALPHA8_ASTC_10x8_KHR = 0x93DA
    COMPRESSED_SRGB8_ALPHA8_ASTC_10x10_KHR = 0x93DB
    COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR = 0x93DC
    COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR = 0x93DD



# Copyright (c) 2023, Shukant Pal and Contributors
# SPDX-License-Identifier: Apache-2.0

from dataclasses import dataclass
from .gl_internalformat import GlInternalformat
from typing import Optional
from .vk_format import VkFormat


@dataclass
class KtxTextureCreateInfo:
    """Data for passing texture information to KtxTexture1.create() and KtxTexture2.create()."""

    gl_internal_format: Optional[GlInternalformat]
    """Internal format for the texture, e.g., GlInteralformat.RGB8. Ignored when creating a KtxTexture2."""

    base_width: int
    """Width of the base level of the texture."""

    base_height: int
    """Height of the base level of the texture."""

    base_depth: int
    """Depth of the base level of the texture."""

    vk_format: VkFormat = VkFormat.VK_FORMAT_UNDEFINED
    """VkFormat for texture. Ignored when creating a KtxTexture1."""

    num_dimensions: int = 2
    """Number of dimensions in the texture, 1, 2 or 3."""

    num_levels: int = 1
    """Number of mip levels in the texture. Should be 1 if generateMipmaps is true."""

    num_layers: int = 1
    """Number of array layers in the texture."""

    num_faces: int = 1
    """Number of faces: 6 for cube maps, 1 otherwise."""

    is_array: bool = False
    """Set to true if the texture is to be an array texture. Means OpenGL will use a GL_TEXTURE_*_ARRAY target."""

    generate_mipmaps: bool = False
    """Set to true if mipmaps should be generated for the texture when loading into a 3D API."""
